const path = require('path')
const fs = require('fs-extra')
const chalk = require('chalk')
const inquirer = require('inquirer')  // 交互式问题模块
const semver = require('semver')      // 语义化版本可控制工具

const Creator = require('./creator')

const {
  shouldUseYarn,
  shouldUseCnpm,
  getPkgVersion
} = require('./util')
const { SOURCE_DIR } = require('./config')

class Project extends Creator {
  constructor (options) {
    super()
    const unSupportedVer = semver.lt(process.version, 'v7.6.0')
    if (unSupportedVer) {
      throw new Error('Node.js 版本过低，推荐升级 Node.js 至 v8.0.0+')
    }
    this.rootPath = this._rootPath
    this.conf = Object.assign({
      projectName: null,
      template: null,
      description: ''
    }, options)
  }

  init () {
    console.log(chalk.green(`bee即将创建一个新项目!`))
    console.log()
  }

  create () {
    //NOTE: 此处外部手动调用 create(),将ask的结果加入this.conf,然后调用this.write写入函数
    this.ask()
      .then(answers => {
        console.log(answers)
        console.log(chalk.red(`answers:${answers.toString()}`))
        const date = new Date()
        this.conf = Object.assign(this.conf, answers)
        this.conf.date = `${date.getFullYear()}-${(date.getMonth() + 1)}-${date.getDate()}`
        this.write()
      })
  }

  ask () {
    const prompts = []
    const conf = this.conf
    if (typeof conf.projectName !== 'string') {
      prompts.push({
        type: 'input',
        name: 'projectName',
        message: '请输入项目名称！',
        validate (input) {
          if (!input) {
            return '项目名不能为空！'
          }
          if (fs.existsSync(input)) {
            return '当前目录已经存在同名项目，请换一个项目名！'
          }
          return true
        }
      })
    } else if (fs.existsSync(conf.projectName)) {
      prompts.push({
        type: 'input',
        name: 'projectName',
        message: '当前目录已经存在同名项目，请换一个项目名！',
        validate (input) {
          if (!input) {
            return '项目名不能为空！'
          }
          if (fs.existsSync(input)) {
            return '项目名依然重复！'
          }
          return true
        }
      })
    }

    if (typeof conf.description !== 'string') {
      prompts.push({
        type: 'input',
        name: 'description',
        message: '请输入项目介绍！'
      })
    }

    const cssChoices = [{
      name: 'Sass',
      value: 'sass'
    }, {
      name: 'Less',
      value: 'less'
    }, {
      name: 'Stylus',
      value: 'stylus'
    }, {
      name: '无',
      value: 'none'
    }]

    prompts.push({
      type: 'list',
      name: 'css',
      message: '请选择 CSS 预处理器（Sass/Less/Stylus）',
      choices: cssChoices
    })

    return inquirer.prompt(prompts)
  }

  write (cb) {
    this.conf.src = SOURCE_DIR //资源文件
    this.conf.template = 'redux'
    // NOTE: 此处调用继承过来的Creator的templatePath函数生成模板路径，然后执行模板路径里面的index.js函数
    const templateCreate = require(path.join(this.templatePath(), this.conf.template, 'index.js'))
    templateCreate(this, this.conf, {
      shouldUseYarn,
      shouldUseCnpm,
      getPkgVersion
    }, cb)
  }
}

module.exports = Project
